import { h } from "@stencil/core";
import { watchBlockNumber } from "@wagmi/core";
import { MDCSelect } from "@material/select";
import { BigNumber } from "ethers";
import { handleError } from "../../utils/errors";
import state from "../../stores/wallet";
import { GOERLI_APE_COIN_ADDRESS, MAINNET_APE_COIN_ADDRESS, getAllowance, getBalance, increaseAllowance, } from "../../utils/erc20";
export class NKDropMintButton {
    constructor() {
        this.select = null;
        this.disabled = true;
        this.loading = true;
        this.supply = undefined;
        this.maxAmount = undefined;
        this.maxPerMint = undefined;
        this.price = undefined;
        this.mintFee = undefined;
        this.erc20Price = undefined;
        this.saleActive = undefined;
        this.presaleActive = undefined;
        this.selections = [];
        this.dialogOpen = false;
        this.selectedValue = -1;
        this.successTitle = 'Success';
        this.successMessage = 'Successfully minted an NFT';
    }
    componentWillLoad() {
        var _a;
        this.disconnect = watchBlockNumber({ listen: true, chainId: (_a = state.chain) === null || _a === void 0 ? void 0 : _a.id }, async () => {
            const [supplyValue, maxAmountValue, maxPerMintValue, priceValue, saleActiveValue, presaleActiveValue,] = await Promise.all([
                state.diamond.base.totalSupply(),
                state.diamond.apps.drop.maxAmount(),
                state.diamond.apps.drop.maxPerMint(),
                state.diamond.apps.drop.price(),
                state.diamond.apps.drop.saleActive(),
                state.diamond.apps.drop.presaleActive(),
            ]);
            let erc20Price = BigInt(0);
            let mintFee = BigNumber.from(0);
            let erc20SaleActive = false;
            let erc20PresaleActive = false;
            if (state.diamond.apps.erc20) {
                const [erc20PriceValue, erc20MintFeeValue, erc20SaleActiveValue, erc20PresaleActiveValue,] = await Promise.all([
                    state.diamond.apps.erc20.erc20Price(),
                    state.diamond.apps.erc20.erc20MintFee(),
                    state.diamond.apps.erc20.erc20SaleActive(),
                    state.diamond.apps.erc20.erc20PresaleActive(),
                ]);
                erc20Price = BigInt(Number(erc20PriceValue));
                mintFee = erc20MintFeeValue;
                erc20SaleActive = erc20SaleActiveValue;
                erc20PresaleActive = erc20PresaleActiveValue;
            }
            else if (state.diamond.apps.ape) {
                const [apePriceValue, apeMintFeeValue, apeSaleActiveValue, apePresaleActiveValue,] = await Promise.all([
                    state.diamond.apps.ape.apePrice(),
                    state.diamond.apps.ape.apeMintFee(),
                    state.diamond.apps.ape.apeSaleActive(),
                    state.diamond.apps.ape.apePresaleActive(),
                ]);
                erc20Price = BigInt(Number(apePriceValue));
                mintFee = apeMintFeeValue;
                erc20SaleActive = apeSaleActiveValue;
                erc20PresaleActive = apePresaleActiveValue;
            }
            this.erc20Price = erc20Price;
            this.mintFee = mintFee;
            this.supply = Number(supplyValue);
            this.maxAmount = Number(maxAmountValue);
            this.maxPerMint = Number(maxPerMintValue);
            this.price = priceValue;
            this.saleActive = saleActiveValue || erc20SaleActive;
            this.presaleActive = presaleActiveValue || erc20PresaleActive;
            this.selections = Array.from({ length: this.maxPerMint }, (_, i) => i + 1);
            this.loading = false;
            // sale not active then disable widget
            this.disabled = !(this.saleActive || this.presaleActive);
        });
    }
    componentDidLoad() {
        this.setSelectedText();
    }
    componentDidUpdate() {
        if (this.selections.length > 0 && !this.select) {
            this.select = new MDCSelect(this.container);
            this.select.listen('MDCSelect:change', () => {
                // trigger mint
                this.selectedValue = Number(this.select.value);
                this.mint(this.selectedValue);
            });
        }
        this.setSelectedText();
    }
    disconnectedCallback() {
        this.disconnect();
    }
    async mint(quantity) {
        var _a, _b, _c, _d;
        try {
            this.loading = true;
            const address = (_b = (_a = state.walletClient) === null || _a === void 0 ? void 0 : _a.account) === null || _b === void 0 ? void 0 : _b.address;
            const chainId = await ((_c = state.walletClient) === null || _c === void 0 ? void 0 : _c.getChainId());
            if (chainId !== ((_d = state.chain) === null || _d === void 0 ? void 0 : _d.id)) {
                state.modal.open({
                    view: 'Networks',
                });
                return;
            }
            if (this.presaleActive) {
                const verify = await state.diamond.verify(address);
                if (state.diamond.apps.ape) {
                    const erc20Contract = state.isDev
                        ? GOERLI_APE_COIN_ADDRESS
                        : MAINNET_APE_COIN_ADDRESS;
                    await this.ensureERC20Allowance(quantity, erc20Contract, state.diamond.apps.ape.address);
                    const tx = await state.diamond.apps.ape.apePresaleMintTo(address, quantity, verify.allowed, verify.proof, {
                        value: this.mintFee.mul(quantity),
                    });
                    await tx.wait();
                    this.dialogTitle = this.successTitle;
                    this.dialogMessage = this.successMessage;
                    this.dialogOpen = true;
                    return;
                }
                if (state.diamond.apps.erc20) {
                    const erc20Contract = (await state.diamond.apps.erc20.erc20ActiveCoin());
                    await this.ensureERC20Allowance(quantity, erc20Contract, state.diamond.apps.erc20.address);
                    const tx = await state.diamond.apps.erc20.erc20PresaleMintTo(address, quantity, verify.allowed, verify.proof, {
                        value: this.mintFee.mul(quantity),
                    });
                    await tx.wait();
                    this.dialogTitle = this.successTitle;
                    this.dialogMessage = this.successMessage;
                    this.dialogOpen = true;
                    return;
                }
                const tx = await state.diamond.apps.drop.presaleMintTo(address, quantity, verify.allowed, verify.proof, {
                    value: this.price.mul(quantity),
                });
                await tx.wait();
                this.dialogTitle = this.successTitle;
                this.dialogMessage = this.successMessage;
                this.dialogOpen = true;
                return;
            }
            if (this.saleActive) {
                if (state.diamond.apps.ape) {
                    const erc20Contract = state.isDev
                        ? GOERLI_APE_COIN_ADDRESS
                        : MAINNET_APE_COIN_ADDRESS;
                    await this.ensureERC20Allowance(quantity, erc20Contract, state.diamond.apps.ape.address);
                    const tx = await state.diamond.apps.ape.apeMintTo(address, quantity, {
                        value: this.mintFee.mul(quantity),
                    });
                    await tx.wait();
                    this.dialogTitle = this.successTitle;
                    this.dialogMessage = this.successMessage;
                    this.dialogOpen = true;
                    return;
                }
                if (state.diamond.apps.erc20) {
                    const erc20Contract = (await state.diamond.apps.erc20.erc20ActiveCoin());
                    await this.ensureERC20Allowance(quantity, erc20Contract, state.diamond.apps.erc20.address);
                    const tx = await state.diamond.apps.erc20.erc20MintTo(address, quantity, {
                        value: this.mintFee.mul(quantity),
                    });
                    await tx.wait();
                    this.dialogTitle = this.successTitle;
                    this.dialogMessage = this.successMessage;
                    this.dialogOpen = true;
                    return;
                }
                const tx = await state.diamond.apps.drop.mintTo(address, quantity, {
                    value: this.price.mul(quantity),
                });
                await tx.wait();
                this.dialogTitle = this.successTitle;
                this.dialogMessage = this.successMessage;
                this.dialogOpen = true;
                return;
            }
            throw new Error('Sale is not active.');
        }
        catch (err) {
            console.log(err);
            this.dialogTitle = 'Error';
            this.dialogMessage = handleError(err);
            this.dialogOpen = true;
        }
        finally {
            this.loading = false;
        }
    }
    async ensureERC20Allowance(quantity, erc20Contract, spenderAddress) {
        var _a, _b;
        const userAddress = (_b = (_a = state.walletClient) === null || _a === void 0 ? void 0 : _a.account) === null || _b === void 0 ? void 0 : _b.address;
        if (!userAddress) {
            throw new Error('Wallet not connected');
        }
        const price = this.erc20Price * BigInt(quantity);
        // check user balance first before getting allowance.
        const balance = await getBalance(state.publicClient, erc20Contract, userAddress);
        if (balance < price) {
            throw new Error('Insufficient balance.');
        }
        // check for allowance
        const currentAllowance = await getAllowance(state.publicClient, erc20Contract, userAddress, spenderAddress);
        if (currentAllowance < price) {
            await increaseAllowance(state.publicClient, state.walletClient, erc20Contract, spenderAddress, price);
        }
    }
    setSelectedText() {
        setTimeout(() => {
            const result = this.selectedValue < 0 ? '<slot />' : this.selectedValue.toString();
            this.selectedText.innerHTML = this.loading ? '<nk-loading />' : result;
        }, 10);
    }
    optionClasses(value) {
        return `mdc-deprecated-list-item ${this.selectedValue === value ? 'mdc-deprecated-list-item--selected' : ''}`;
    }
    render() {
        return (h("div", { part: "mint-btn-container", class: `mdc-select mdc-select--filled mdc-select--no-label ${this.disabled ? 'mdc-select--disabled' : ''}`, ref: (el) => (this.container = el) }, h("div", { part: "mint-btn", class: "mdc-select__anchor", role: "button", "aria-haspopup": "listbox", "aria-expanded": "false", "aria-labelledby": "mint-selected-text" }, h("span", { class: "mdc-select__ripple" }), h("span", { class: "mdc-select__selected-text-container" }, h("span", { part: "mint-text", id: "mint-selected-text", class: "mdc-select__selected-text", ref: (el) => (this.selectedText = el) })), h("span", { part: "mint-dropdown-icon", class: "mdc-select__dropdown-icon" }, h("svg", { class: "mdc-select__dropdown-icon-graphic", viewBox: "7 10 10 5", focusable: "false" }, h("polygon", { class: "mdc-select__dropdown-icon-inactive", stroke: "none", "fill-rule": "evenodd", points: "7 10 12 15 17 10" }), h("polygon", { class: "mdc-select__dropdown-icon-active", stroke: "none", "fill-rule": "evenodd", points: "7 15 12 10 17 15" })))), h("div", { class: "mdc-select__menu mdc-menu mdc-menu-surface mdc-menu-surface--fullwidth" }, h("ul", { class: "mdc-deprecated-list", role: "listbox", "aria-label": "Quantity Picker listbox" }, this.selections.map((value) => (h("li", { class: this.optionClasses(value), "aria-selected": value === this.selectedValue, "data-value": value, role: "option" }, h("span", { class: "mdc-deprecated-list-item__ripple" }), h("span", { class: "mdc-deprecated-list-item__text" }, value)))))), h("nk-dialog", { open: this.dialogOpen, dialogTitle: this.dialogTitle }, this.dialogMessage)));
    }
    static get is() { return "nk-drop-mint-button"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["nk-drop-mint-button.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["nk-drop-mint-button.css"]
        };
    }
    static get properties() {
        return {
            "successTitle": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Title on the success modal"
                },
                "attribute": "success-title",
                "reflect": false,
                "defaultValue": "'Success'"
            },
            "successMessage": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "Body message on the success modal"
                },
                "attribute": "success-message",
                "reflect": false,
                "defaultValue": "'Successfully minted an NFT'"
            }
        };
    }
    static get states() {
        return {
            "disabled": {},
            "loading": {},
            "supply": {},
            "maxAmount": {},
            "maxPerMint": {},
            "price": {},
            "mintFee": {},
            "erc20Price": {},
            "saleActive": {},
            "presaleActive": {},
            "selections": {},
            "dialogOpen": {},
            "selectedValue": {}
        };
    }
    static get methods() {
        return {
            "mint": {
                "complexType": {
                    "signature": "(quantity: number) => Promise<void>",
                    "parameters": [{
                            "tags": [],
                            "text": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            }
        };
    }
}
//# sourceMappingURL=nk-drop-mint-button.js.map
