import Diamond from "@niftykit/diamond";
import { Env } from "@stencil/core";
import { createStore } from "@stencil/store";
import { arbitrum, arbitrumGoerli, arbitrumNova, goerli, mainnet, optimism, optimismGoerli, polygon, polygonMumbai, base, baseGoerli, avalanche, avalancheFuji, } from "@wagmi/chains";
import { watchWalletClient } from "@wagmi/core";
import { createWeb3Modal, defaultWagmiConfig } from "@web3modal/wagmi";
import { publicClientToProvider, walletClientToSigner, } from "../utils/adapters";
const projectId = Env.projectId;
const availableChains = [
    mainnet,
    goerli,
    polygon,
    polygonMumbai,
    arbitrum,
    arbitrumNova,
    arbitrumGoerli,
    optimism,
    optimismGoerli,
    base,
    baseGoerli,
    avalanche,
    avalancheFuji,
];
const { state } = createStore({});
export async function initialize(collectionId, isDev) {
    var _a;
    const data = await Diamond.getCollectionData(collectionId, isDev);
    if (!data) {
        throw new Error('Invalid collection.');
    }
    const metadata = {
        name: 'NiftyKit',
        description: 'Sign Up for a NiftyKit account and create your first digital collectible in just minutes, instead of hours and days.',
        url: 'https://app.niftykit.com',
        icons: ['https://app.niftykit.com/logo-new.svg'],
    };
    const chains = availableChains.filter((chain) => chain.id === data.chainId);
    const wagmiConfig = defaultWagmiConfig({ chains, projectId, metadata });
    state.isDev = isDev;
    state.publicClient = wagmiConfig.publicClient;
    state.modal = createWeb3Modal({
        wagmiConfig,
        projectId,
        chains,
        themeVariables: {
            '--w3m-font-family': 'Chivo, sans-serif',
            '--w3m-z-index': 99999,
        },
        termsConditionsUrl: 'https://niftykit.notion.site/Terms-of-Use-d81954b4fbaf4d5da2f52a39722d532a',
        privacyPolicyUrl: 'https://niftykit.notion.site/Privacy-Policy-6afe9633a54b4119b66a3ea1ab79cf50',
    });
    state.chain = chains[0];
    state.diamond = await Diamond.create(publicClientToProvider((_a = wagmiConfig.webSocketPublicClient) !== null && _a !== void 0 ? _a : wagmiConfig.publicClient), collectionId, data, isDev);
    watchWalletClient({
        chainId: data.chainId,
    }, async (walletClient) => {
        state.diamond = await Diamond.create(walletClientToSigner(walletClient), collectionId, data, isDev);
        state.walletClient = walletClient;
    });
}
export default state;
//# sourceMappingURL=wallet.js.map
